//+------------------------------------------------------------------+
//|                                                  TimeLibrary.mq4 |
//|                        Copyright 2016, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property library
#property copyright "Copyright 2016, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property strict


#import "kernel32.dll"
int SystemTimeToFileTime(int& TimeArray[], int& FileTimeArray[]);
int FileTimeToLocalFileTime(int& FileTimeArray[], int& LocalFileTimeArray[]);
void GetSystemTime(int& TimeArray[]);
#import

datetime GetLocalDateTime() export
{
   double hundrednSecPerSec = 10.0 * 1000000.0;
   double bit32to64 = 65536.0 * 65536.0;
   double secondsBetween1601And1970 = 11644473600.0;
   int    TimeArray[4];
   int    FileTimeArray[2];   // 100nSec since 1601/01/01 UTC
   int    LocalFileTimeArray[2];   // 100nSec since 1601/01/01 Local

   GetSystemTime(TimeArray);
   SystemTimeToFileTime(TimeArray, FileTimeArray);
   FileTimeToLocalFileTime(FileTimeArray, LocalFileTimeArray);
   
   double lfLo32 = LocalFileTimeArray[0];
   if(lfLo32 < 0)
      lfLo32 = bit32to64 + lfLo32;
   double ticksSince1601 = LocalFileTimeArray[1] * bit32to64 + lfLo32;
   double secondsSince1601 = ticksSince1601 / hundrednSecPerSec;
   double secondsSince1970 = secondsSince1601 - secondsBetween1601And1970;
   return ((datetime) secondsSince1970);
}

datetime GetUTCDateTime() export
{
   double hundrednSecPerSec = 10.0 * 1000000.0;
   double bit32to64 = 65536.0 * 65536.0;
   double secondsBetween1601And1970 = 11644473600.0;
   int    TimeArray[4];
   int    FileTimeArray[2];   // 100nSec since 1601/01/01 UTC
   GetSystemTime(TimeArray);
   SystemTimeToFileTime(TimeArray, FileTimeArray);
   
   double lfLo32 = FileTimeArray[0];
   if(lfLo32 < 0)
      lfLo32 = bit32to64 + lfLo32;
   double ticksSince1601 = FileTimeArray[1] * bit32to64 + lfLo32;
   double secondsSince1601 = ticksSince1601 / hundrednSecPerSec;
   double secondsSince1970 = secondsSince1601 - secondsBetween1601And1970;
   return ((datetime)secondsSince1970);
}

void PauseInMilliSeconds(uint pause) export
{
uint k,n;

k=GetUTCMilliSeconds();
n=k+pause;

while(k<n && !IsStopped()) 
  { 
    k=GetUTCMilliSeconds();
  }
}

void PauseInTicks(uint pause) export
{
uint k,n;

k=GetUTCTicks();
n=k+pause;

while(k<n && !IsStopped()) 
  { 
    k=GetUTCTicks();
  }
}

void PauseInMicroSeconds(uint pause) export
{
uint k,n;

k=GetUTCMicroSeconds();
n=k+pause;

while(k<n && !IsStopped()) 
  { 
    k=GetUTCMicroSeconds();
  }
}

uint GetUTCMilliSeconds()
{
   double hundrednSecPerSec = 10.0 * 1000.0;
   double bit32to64 = 65536.0 * 65536.0;
   //double secondsBetween1601And1970 = 11644473600.0;
   int    TimeArray[4];
   int    FileTimeArray[2];   // 100nSec since 1601/01/01 UTC
   GetSystemTime(TimeArray);
   SystemTimeToFileTime(TimeArray, FileTimeArray);
   
   double lfLo32 = FileTimeArray[0];
   if(lfLo32 < 0)
      lfLo32 = bit32to64 + lfLo32;
   double ticksSince1601 = FileTimeArray[1] * bit32to64 + lfLo32;
   double secondsSince1601 = ticksSince1601 / hundrednSecPerSec;
   //double secondsSince1970 = secondsSince1601 - secondsBetween1601And1970;
   return ((uint)secondsSince1601);
}

uint GetUTCMicroSeconds()
{
   double hundrednSecPerSec = 10.0;
   double bit32to64 = 65536.0 * 65536.0;
   //double secondsBetween1601And1970 = 11644473600.0;
   int    TimeArray[4];
   int    FileTimeArray[2];   // 100nSec since 1601/01/01 UTC
   GetSystemTime(TimeArray);
   SystemTimeToFileTime(TimeArray, FileTimeArray);
   
   double lfLo32 = FileTimeArray[0];
   if(lfLo32 < 0)
      lfLo32 = bit32to64 + lfLo32;
   double ticksSince1601 = FileTimeArray[1] * bit32to64 + lfLo32;
   double secondsSince1601 = ticksSince1601 / hundrednSecPerSec;
   //double secondsSince1970 = secondsSince1601 - secondsBetween1601And1970;
   return ((uint)secondsSince1601);
}

uint GetUTCTicks()
{
   double hundrednSecPerSec = 1;
   double bit32to64 = 65536.0 * 65536.0;
   //double secondsBetween1601And1970 = 11644473600.0;
   int    TimeArray[4];
   int    FileTimeArray[2];   // 100nSec since 1601/01/01 UTC
   GetSystemTime(TimeArray);
   SystemTimeToFileTime(TimeArray, FileTimeArray);
   
   double lfLo32 = FileTimeArray[0];
   if(lfLo32 < 0)
      lfLo32 = bit32to64 + lfLo32;
   double ticksSince1601 = FileTimeArray[1] * bit32to64 + lfLo32;
   double secondsSince1601 = ticksSince1601 / hundrednSecPerSec;
   //double secondsSince1970 = secondsSince1601 - secondsBetween1601And1970;
   return ((uint)secondsSince1601);
}